/******************************************************************************
modified by Ralf Steinert for LPC213x
 *****************************************************************************/

#include "LPC213x.h"
#include "config.h"
#include "uart.h"

#define BAUD 9600

// Keil MCB2130 (LPC2138): one led on P1.23 (active low)
#define LEDPIN  23

void systemInit(void)
{
	// --- enable and connect the PLL (Phase Locked Loop) ---
	// a. set multiplier and divider
	PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	// b. enable PLL
	PLLCON = (1<<PLLE);
	// c. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	// d. wait for PLL lock (PLOCK bit is set if locked)
	while (!(PLLSTAT & (1<<PLOCK)));
	// e. connect (and enable) PLL
	PLLCON = (1<<PLLE) | (1<<PLLC);
	// f. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	
	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAMCR = 0;	
	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMTIM = MAM_FETCH;
	// c. enable MAM 
	MAMCR = MAM_MODE;
	
	// --- set VPB speed ---
	VPBDIV = VPBDIV_VAL;
	
	// --- map INT-vector ---
	#if defined(RAM_RUN)
	  MEMMAP = MEMMAP_USER_RAM_MODE;
	#elif defined(ROM_RUN)
	  MEMMAP = MEMMAP_USER_FLASH_MODE;
	#else
	#error RUN_MODE not defined!
	#endif
}

void gpioInit(void)
{
	IODIR1 = 0xFF0000;					// define P1.16 - P1.23 as output
	IOSET1  = (0xff);					// set all LED off (active low)
}

void ledToggle(void)
{
	static unsigned char state=0;
	
	state = !state;
	if (state) IOCLR1 = (1<<LEDPIN);	// set Bit = LED on
	else IOSET1 = (1<<LEDPIN);			// set Bit = LED off (active low)
}

int main(void)
{
	int ch;
	
	systemInit();
	gpioInit();
	
	uart0Init(UART_BAUD(BAUD), UART_8N1, UART_FIFO_8); // setup the UART
	
	uart0Putch('M');
	
	uart0Puts("\r\nHello from WinARM!\r\n");
	uart0Puts("Demo based on code from R O Software\r\n");
	
	ledToggle();
	
	while(1) {
		if ((ch = uart0Getch()) >= 0) {
			uart0Puts("You pressed : ");
			uart0Putch(ch);
			uart0Puts("\r\n");
			ledToggle();
		}
	}
	
	return 0;
}
